<?php
namespace PME\Controllers\Relatorios;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class EmissaoCardPeriodoPAEController extends \Zeedhi\Framework\Controller\Report {

  protected $entityManager;
  protected $birtStrategy;
  protected $newCode;
  protected $crudService;
  protected $utilities;

  public function __construct(ReportService $reportService, $entityManager, $birtStrategy, $newCode, $crudService, $utilities) {
      $this->entityManager = $entityManager;
      $this->birtStrategy  = $birtStrategy;
      $this->newCode       = $newCode;
      $this->crudService   = $crudService;
      $this->utilities     = $utilities;
      

      parent::__construct($reportService);
  }

  protected $reportMapping = array (
    'PME01300' => 
      array (
          'strategy' => 'BIRT',
          'parameterMapping' => 
          array (
            "CDFILCENTPME"  => "P_CDFILCENTPME",
            "NMFILCENTPME"  => "P_NMFILCENTPME",
            "NRSEQTRABCARD" => "P_NRSEQTRABCARD",
            "PERIODODE"     => "P_DTINI",
            "PERIODOATE"    => "P_DTFIN",
            "NRLISTASERV"   => "P_NRLISTASERV",
            "DSCARDPME"     => "P_DSCARDPME",
            "NMFILIAGRE"    => "P_NMFILIAGRE",
            "CDFILIAL"      => "P_CDFILIAL",
            "NRORG"         => "P_NRORG"
          )
      )
  );
  
  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $row = $request->getRow();
      
      $nrorg = $this->utilities->getNrOrg();
      $row['NRSEQTRABCARD'] = $this->newCode->getCodeByTableName('TRABCARDMES', $nrorg);
      $this->setParams($row);

      $row['NRORG'] = $nrorg;
      
      $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
  }
  
  private function setParams($row) {
    $servicos = '';
    $servicosLista = '';
    $arrayServLista = [];

    $tiposDePratos = '';

    $lastServ = null;
    $count = 0;

    $numServ = 0;

    if(empty($row['TIPOPRAT'])) $tipoPrat = $this->getTpPrato($row, null);
    else {
      $tpPratFilter = '';
      foreach($row['TIPOPRAT'] as $tipo) {
        $tpPratFilter .= $tipo['CDSERVFILI'].$tipo['CDTIPOPRAT'];
      }
      $tipoPrat = $this->getTpPrato($row, $tpPratFilter);
    }
    $this->validServsCard($row, $tipoPrat);

    foreach($tipoPrat as $item){
      if($lastServ != $item['CDSERVFILI'] || $count == 4){ 
        if($lastServ != null) $tiposDePratos .= '|';
        $servicos .= $item['CDSERVFILI'].'_';
        $servicosLista .= $item['CDSERVFILI'].'_';
        $lastServ = $item['CDSERVFILI'];
        
        $numServ ++;
        $count = 0;

        if($numServ == 5) {
          array_push($arrayServLista, $servicosLista);
          $numServ = 0;
          $servicosLista = '';
        }
      }
      
      $tiposDePratos .= $item['CDTIPOPRAT'].':'.$item['NRORDCNFCPME'].'_';
      $count ++;
    }

    // insere serviços que não excederam 5 registros
    if($numServ != 5) array_push($arrayServLista, $servicosLista);

    $row['NRLISTASERV'] = $this->insereLista($arrayServLista);

    $cdServicos = explode('_', $servicos);
    $this->procedure($row, $servicos, $tiposDePratos);
  }

  private function getTpPrato($row, $tpPratFilter) {
    $conditions = [
      array(
        "columnName" => 'P_NRCARDPME',
        "operator"   => '=',
        "value"      => $row['NRCARDPME']
      ),
      array(
        "columnName" => 'P_CDSERVFILI',
        "operator"   => '=',
        "value"      => empty($row['CDSERVFILI']) ? null : join('_', $row['CDSERVFILI'])
      ),
      array(
        "columnName" => 'P_CDFILCENTPME',
        "operator"   => '=',
        "value"      => $row['CDFILCENTPME']
      ),
      array(
        "columnName" => 'P_SERVTPPRAT',
        "operator"   => '=',
        "value"      => $tpPratFilter
      )
    ];
    $dataSet = $this->crudService->getFilteredDataSet("tipoPratoConfigCardPorServ", $conditions);  
    return $dataSet->getRows();
  }

  private function procedure($row, $servicos, $tiposDePratos){
    
    $nrorg = $this->utilities->getNrOrg();

    if(empty($row['CDFILIAL'])){
      $cdfilial = $row['CDFILCENTPME'];
    } else $cdfilial = $row['CDFILIAL'];
      
    $connection = $this->entityManager->getConnection();
    $procedureCons = new StoredProcedure($connection, 'TRABRELCARDPER_PME');
    
    $procedureCons->addParam(new Param('P_NRSEQTRABCARD', Param::PARAM_INPUT, $row['NRSEQTRABCARD'], Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDFILCENTPME',  Param::PARAM_INPUT, $row['CDFILCENTPME'],  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDFILIAL',      Param::PARAM_INPUT, $cdfilial,             Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDTIPOCARD',    Param::PARAM_INPUT, $row['CDTIPOCARD'],    Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_NRCARDPME',     Param::PARAM_INPUT, $row['NRCARDPME'],     Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $row['PERIODODE'],     Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $row['PERIODOATE'],    Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDSERVFILI',    Param::PARAM_INPUT, $servicos,             Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDTIPOPRAT',    Param::PARAM_INPUT, $tiposDePratos,        Param::PARAM_TYPE_STR));
    $procedureCons->execute();
  }

  private function insereLista($arrayServLista) {
    $nrorg = $this->utilities->getNrOrg();
    $nrlista = $this->newCode->getCodeByTableName('lista', $nrorg);
    $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";

    foreach ($arrayServLista as $servs) {
      if(!empty($servs)){
        $stmt = $this->entityManager->getConnection()->prepare($sql);
        $stmt->execute(
            array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $servs,
                "NRORG" => $nrorg
            )
        );
      }
    }

    return $nrlista;
  }

  private function validServsCard($row, &$tipoPrat) {
    $servs = [];
    $newTipoPrat = [];
    
    $query = 'SELECT DISTINCT C.CDSERVFILI FROM CARDAPIOPME CARD
              INNER JOIN CARDIAPME C ON C.CDFILIAL = NVL(:CDFILIAL, :CDFILCENTPME) AND C.DTCARDIAPME BETWEEN :DTINI AND :DTFIN AND C.NRCARDPME = CARD.NRCARDPME AND C.CDFILCENTPME = CARD.CDFILCENTPME
              WHERE CARD.CDFILCENTPME = :CDFILCENTPME AND CARD.CDTIPOCARD = :CDTIPOCARD';
    $params = array(
      "CDFILCENTPME"  => $row['CDFILCENTPME'],
      "CDTIPOCARD"    => $row['CDTIPOCARD'],
      "CDFILIAL"      => $row['CDFILIAL'],
      "DTINI"         => $row['PERIODODE'],
      "DTFIN"         => $row['PERIODOATE']
    );
    
    $data = $this->entityManager->getConnection()->fetchAll($query, $params);
    
    foreach($data as $item) array_push($servs, $item['CDSERVFILI']);

    foreach($tipoPrat as $item) {
      if(in_array($item['CDSERVFILI'], $servs)) array_push($newTipoPrat, $item);
    }

    $tipoPrat = $newTipoPrat;
  }
}