<?php

/**
 * Contribuicao Sindical.
 *
 * @package Logic
 *
 * @author  alessandrobartels
 *
 */
class ContribuicaoSindical{


    /**
     * Calcula Contribuicao Sindical.
     *
     * @param integer               $evento          Numero do evento.
     * @param TkDateTime            $mes_competencia Mes de competencia.
     * @param FpaitecalcfolhaEntity $item_de_calculo Itens de calculo.
     * @param FpaassoceventoEntity  $assoc_evento    Associacao de eventos.
     * @param FpaeventomEntity      $eventom         Evento.
     * @param GpevinculomEntity     $vinculo         Vinculo.
     *
     * @return void
     */
    public static function calcula(
        $evento, TkDateTime $mes_competencia, FpaitecalcfolhaEntity $item_de_calculo, FpaassoceventoEntity $assoc_evento,
        FpaeventomEntity $eventom, GpevinculomEntity $vinculo
    ){
        $id_contribuicao_sindical = $vinculo->historico->getIdcontribuisind();
        $desconto = false;

        switch($id_contribuicao_sindical){
            /* Ja descontou num emprego anterior e deve virar "SEMPRE" em janeiro do proximo ano*/
        case GpevinculohEntityBase::IDCONTRIBUISIND_DESCONTOU:
        case GpevinculohEntityBase::IDCONTRIBUISIND_NUNCA:
                return;
        case GpevinculohEntityBase::IDCONTRIBUISIND_SEMPRE:
                $dt_admissao = new TkDateTime($vinculo->getDtadmissaovinc(), TkDateTime::FORMAT_DATE);
                $data_corrente = new TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
                $dt_mes_anterior = new TkDateTime($data_corrente, TkDateTime::FORMAT_DATE);
                $mes = $data_corrente->toValue(TkDateTime::MONTH);
                $dt_mes_anterior->subMonth(1)->subDay(1);

            if (($mes === 3) && (($dt_admissao->isEarlier($data_corrente)) === true)) {
                /* Mes = marco e qualquer data de admissao anterior*/
                $desconto = true;
            } elseif (($mes === 1) && (($dt_admissao->isLater($dt_mes_anterior)) === true)
                                   && (($dt_admissao->isEarlier($data_corrente)) === true)) {
                /* Mes = janeiro e admissao em dezembro do ano anterior*/
                $desconto = true;
            } elseif (($mes >= 4) && (($dt_admissao->isLater($dt_mes_anterior)) === true)
                                 && (($dt_admissao->isEarlier($data_corrente)) === true)) {
                /* Mes >= abril e admissao no mes anterior */
                $desconto = true;
            } else {
                $desconto = false;
            }

            if ($desconto === true) {
                $nrevento =  $assoc_evento->retornaEvento($evento, $vinculo->getNrorg());
                $evento_contrib_sind = $eventom->retornaEvento($nrevento);

                if ($evento_contrib_sind->historico->getIdunidrefereven() !== FpaeventohEntity::IDUNIDREFEREVEN_DIA) {
                    /* @todo Converter para tk_show_message */
                    //@todo MsgCalculo
                    MsgCalculo::getInstance()->adicionaMensagem('Referência de contribuicao sindical precisa ser dia.');
                    //echo '<<CONTRIBUICAO SINDICAL>>Referencia de contribuicao sindical precisa ser dia.<br/>';
                }

                /* @todo Tratar referencia em horas */
                $referencia = 1; /* 1 dia de trabalho */
                // $item_de_calculo->geraEvento($evento_contrib_sind, $mes_competencia, 0, $referencia, 1);.
                $item_de_calculo->geraEvento($evento_contrib_sind, null, 0, $referencia, 1);
            }

            break;
        }

    }


}